<?

	class axisAPI {
		
		private $serverDomain		= "";
		private $apiKey				= "";
		
		private $useBearerTokens	= false;
		private $bearerToken		= null;
		
		private $field_filters		= [];
		private $user_filters		= [];
		
		private $user_agent			= "AxisAPI PHP Client v1.0.0";
		
		
		public function __construct () {			
			if ( !in_array('curl', get_loaded_extensions()) ) {
				throw new Exception('Axis API Error: CURL PHP extension not installed or unavailable.');
			}
		}
		
		
		public function setServerDomain ( $in_domain ) {
			
			$in_domain	= trim($in_domain);
			$in_domain 	= strtolower($in_domain);
			$in_domain	= str_replace("http://", "", $in_domain);
			$in_domain	= str_replace("https://", "", $in_domain);
			
			if ( substr($in_domain, -1) == "/" ) {
				$in_domain = substr($in_domain, 0, -1);
			}
			
			$this->serverDomain = $in_domain;
		}
			
		
		public function setAPIKey ( $in_apiKey ) {
			
			$in_apiKey = trim($in_apiKey);
			
			$this->apiKey = $in_apiKey;
		}
		
		
		public function useBearerTokens () {
			
			// only request a bearer token if we don't already have one
			//
			if ( $this->bearerToken == "" ) {
				
				$result = $this->get("/token/request/");

				if ( $result->success ) {
					
					$json_response 		= json_decode($result->json, true);
					$this->bearerToken	= trim($json_response['token']);
					
				} else {
					$json_response 		= json_decode($result->json, true);
					throw new Exception('Axis API Error: ' . $json_response['error'] . " - " . $json_response['message']);
				}
			}
			
			$this->useBearerTokens = true;
			
		}
		
		
		public function revokeBearerToken () {
			if ( $this->bearerToken != "" ) {
				$result = $this->get("/token/revoke/?token=" . $this->bearerToken );
			}
		}
		
		public function clearFilters () {
			$this->field_filters = [];
			$this->user_filters = [];
		}
		
		
		public function addFieldFilter ( $in_field_name ) {
			
			if ( is_array($in_field_name) ) {
				
				foreach ( $in_field_name as $key => $field_name ) {
					$field_name	= trim(strtolower($field_name));
					
					if ( $field_name != "" ) { 
						$this->field_filters[] = $field_name;
					}
				}
				
			} else {
			
				$in_field_name	= trim(strtolower($in_field_name));

				if ( $in_field_name == "" ) {
					throw new Exception('Axis API Error: Field Name Missing');
				}

				$this->field_filters[] = $in_field_name;
			}			
		}
		
		
		public function clearFieldFilter () {
			$this->field_filters = [];
		}
		
		
		public function addUserFilter ( $in_field_name, $in_comparison, $in_value ) {
			
			$in_field_name	= trim(strtolower($in_field_name));
			$in_comparison	= trim($in_comparison);
			$in_value		= trim($in_value);
			
			if ( $in_field_name == "" ) {
				throw new Exception('Axis API Error: Field Name Missing');
			}
			
			if ( 		$in_comparison == "equals" 		|| $in_comparison == "=" ) 	{ 	$in_comparison = "="; 
			} else if ( $in_comparison == "contains" 	|| $in_comparison == "~" ) 	{ 	$in_comparison = "~"; 
			} else if ( $in_comparison == "not" 		|| $in_comparison == "!=" ) { 	$in_comparison = "!="; 
			} else if ( $in_comparison == "excludes" 	|| $in_comparison == "!~" )	{ 	$in_comparison = "!~"; 
			} else {
				throw new Exception('Axis API Error: Field Comparison Operator Missing');
			}
			
			$this->user_filters[] = ["field" => $in_field_name, "comparison" => $in_comparison, "value" => $in_value];
		}
		
		public function clearUserFilter () {
			$this->user_filters = [];
		}
		
		public function get ( $in_endpoint ) {
			return $this->callAPI($in_endpoint);
		}
		
		
		public function post ( $in_endpoint, $in_parameters = [] ) {
			return $this->callAPI($in_endpoint, "POST", $in_parameters);
		}
		
		
		public function put ( $in_endpoint, $in_parameters = [] ) {
			return $this->callAPI($in_endpoint, "PUT", $in_parameters);
		}
		
		
		public function delete ( $in_endpoint ) {
			return $this->callAPI($in_endpoint, "DELETE");
		}
		
		
		private function callAPI ( $in_endpoint, $in_method = "GET", $in_parameters = [] ) {
			
			// is a server domain set?
			//
			if ( $this->serverDomain == "" ) {
				throw new Exception('Axis API Error: Server Domain Not Set.');
			}
			
			
			// is an API key set?
			//
			if ( $this->apiKey == "" ) {
				throw new Exception('Axis API Error: API Key Not Set.');
			}
			
			
			// if we are in bearer token mode, is a bearer token set?
			//
			if ( $this->useBearerTokens && $this->bearerToken == "" ) {
				throw new Exception('Axis API Error: Bearer Token Not Set.');
			}
			
			
			// do we have an endpoint?
			//
			$in_endpoint = trim($in_endpoint);
			
			if ( $in_endpoint == "" ) {
				throw new Exception('Axis API Error: API Endpoint Not Specified.');
			}
			
			
			// if we get to this point we can attempt the request
			//
			$in_endpoint = strtolower($in_endpoint);
			
			if ( substr($in_endpoint, 0, 1) != "/" ) {
				$in_endpoint = "/" . $in_endpoint;
			}
			
			if ( substr($in_endpoint, -1) != "/" ) {
				$in_endpoint = $in_endpoint . "/";
			}
			
			$endpoint_url = "https://" . $this->serverDomain . "/api.php" . $in_endpoint;
			
			
			// set up default curl options
			//
			$curl_options	= [	CURLOPT_HTTPHEADER => [],
								CURLOPT_HEADER => 0, 
            					CURLOPT_VERBOSE => 1,
							   	CURLOPT_RETURNTRANSFER => TRUE, 
            					CURLOPT_USERAGENT => $this->user_agent,
							  ];
			
			
			// if we're using beater tokens, we pass bearer token in header, otherwise api key
			//
			if ( $this->useBearerTokens ) {
				$curl_options[CURLOPT_HTTPHEADER][] = 'Authorization: Bearer ' . $this->bearerToken;
				
			} else {
				$curl_options[CURLOPT_HTTPHEADER][] = 'APIKEY: ' . $this->apiKey;
			}			
			
			
			// add additional curl options/header/data depending on the method we're using
			//
			if ( $in_method == "POST" ) {
				$curl_options[CURLOPT_CUSTOMREQUEST] = "POST";
				
				if ( sizeof($in_parameters) > 0 ) {
					$curl_options[CURLOPT_POSTFIELDS]	= $in_parameters;
				}
								
			} else if ( $in_method == "PUT" ) {
				$curl_options[CURLOPT_CUSTOMREQUEST] = "PUT";
				
				if ( sizeof($in_parameters) > 0 ) {
					$curl_options[CURLOPT_HTTPHEADER][] = 'Content-Type: application/x-www-form-urlencoded';
					$curl_options[CURLOPT_POSTFIELDS]	= http_build_query($in_parameters);
				}
				
			} else if ( $in_method == "DELETE" ) {
				$curl_options[CURLOPT_CUSTOMREQUEST] = "DELETE";
					
			} else {
				$curl_options[CURLOPT_CUSTOMREQUEST] = "GET";
				
				$field_filter_query	= "";
				$user_filter_query 	= "";
				
				if ( sizeof($this->field_filters) > 0 ) {
					$field_filter_query = "fields=" . urlencode(implode(",", $this->field_filters));										
				}	
				
				if ( sizeof($this->user_filters) > 0 ) {
					$filters = [];
					
					foreach ( $this->user_filters as $key => $filter_data ) {
						$filters[] = implode("", $filter_data);
					}
					
					$user_filter_query = "user_filter=" . urlencode(implode(",", $filters));
										
				}	
				
				if ( $field_filter_query != "" && $user_filter_query != "" ) {
					$endpoint_url .= "?" . $field_filter_query . "&" . $user_filter_query;
					
				} else if ( $field_filter_query != "" ) {
					$endpoint_url .= "?" . $field_filter_query;
					
				} else if ( $user_filter_query != "" ) {
					$endpoint_url .= "?" . $user_filter_query;
				}
			}
			
			
			// make the curl request
			//
			$ch	= curl_init($endpoint_url);		
            curl_setopt_array($ch, $curl_options);      
			curl_setopt($ch, CURLOPT_HEADERFUNCTION, function($curl, $header) use (&$headers) {
				$len = strlen($header);
				$header = explode(':', $header, 2);
				if (count($header) < 2) // ignore invalid headers
					return $len;

				$headers[strtolower(trim($header[0]))][] = trim($header[1]);

				return $len;
			
			});
			
			$response = curl_exec($ch);			
			
			// parse and format the return response
			//
			$apiResponse 	= new stdClass();;
			
			if ( $response !== false ) {
			
				$http_response_code = curl_getinfo($ch, CURLINFO_RESPONSE_CODE);

				if ( $http_response_code == "404" || $http_response_code == "422" || $http_response_code == "429" ) {
					$apiResponse->success = false;
				} else {
					$apiResponse->success = true;
				}

				$apiResponse->timestamp	= date("c");
				$apiResponse->endpoint	= $endpoint_url;
				$apiResponse->headers 	= $headers;
				$apiResponse->json		= $response;
				
			} else {
				$apiResponse->success 	= false;
				$apiResponse->timestamp	= date("c");
				$apiResponse->endpoint	= $endpoint_url;
				$apiResponse->headers	= [];
				$apiResponse->json		= json_encode(["timestamp" => date("c"), "status" => 0, "error" => "API Connection Error", "message" => curl_error($ch)]);
			}
			
			curl_close($ch);
			
			return $apiResponse;
		}
		
	}

?>